-- puzzle

local gr, mg, ky, lm	= love.graphics, love.image, love.keyboard, love.mouse
local pi, sin, cos		= math.pi, math.sin, math.cos

function love.load()
	screen_settings()
	puzzle_settings()
	end

function love.update( dt )
	mouse  = { lm.getX(), lm.getY() }
	mx, my	= locate_cell( mouse )		-- mouse currently over qd(my,mx)
	sx, sy	= locate_cell( sel )		-- selected quad is qd(sy,sx)
	if lm.isDown( 'l' ) then sel = mouse end
	if lm.isDown( 'r' ) then swap_quads() end	-- swap mouse & selected quads
	end

function swap_quads()
	if mx <= steps.x and my <= steps.y then	-- if on image
		qd[sy][sx],		qd[my][mx]		= qd[my][mx],	qd[sy][sx]
		cell[sy][sx],	cell[my][mx]	= cell[my][mx],	cell[sy][sx]
		sel	= mouse
		end
	end

function love.draw()
	draw_quads()
	gr.print( 'fps: '..love.timer.getFPS( ), width - 95, height - 20 )
	if show.grid then grid() end
	if show.text then add_text() end
	show_selection()
	if show.help then							-- overlay image on cells
		gr.setColor( 255, 255, 255, 155 )
		gr.draw( img )
		gr.setColor( 255, 255, 255, 255 )
		end
	end

function love.keypressed( key )
	if key == 'escape'	then love.event.push( 'quit' )	end
	if key == 'right'	then		-- left & right arrow keys rotate quad
		cell[sy][sx][3] = cell[sy][sx][3] < #rot and cell[sy][sx][3] + 1 or 1	end
	if key == 'left'	then
		cell[sy][sx][3] = cell[sy][sx][3] > 1 and cell[sy][sx][3] - 1 or #rot	end
	if key == 'g'		then show.grid	= not show.grid	end
	if key == 'h'		then show.help	= not show.help	end
	if key == 'i'		then show.text	= not show.text	end
	end

function screen_settings()
	colorbg, transbg	= { 0, 0, 180 }, 255
	img 				= gr.newImage( 'Love.jpg')--'test.jpg' ) --'MyTest1.jpg')
	width, height		= img:getWidth(), gr.getHeight()
	show				= {
							grid = true,
							help = false,
							text = true
							}
	text = {
		"left click to select cell",
		"right click to swap cells",
		"< & > to rotate selected cell",
		"<g> to turn off grid",
		"<h> to cheat",
		"<i> to turn off this text",
		"<esc> to exit"
		}
	end

function puzzle_settings()
	gr.setCaption( "Puzzle" )
	cell_size		= 100		-- determines number of squares to be created
	sel				= { 1, 1 }	-- dummy cell selection
	create_quads()
	shuffle_cells()
	randomize_quad_orientations()
	gr.setFont( gr.newFont( 16 ) )
	end

function draw_quads()
	for j = 1, steps.y do
		for i = 1, steps.x do
			gr.drawq(
				img, qd[j][i],					-- source image & quad
				size.x * (i-1), size.y * (j-1),	-- screen position
				rot[cell[j][i][3]][1],			-- quad rotation
				1,1,							-- quad size
				rot[cell[j][i][3]][2],rot[cell[j][i][3]][3] -- quad offsets
				)
			end
		end
	end

function create_quads()
	steps	= { x = width / cell_size,	y = height / cell_size }	-- insures square quads
	size	= { x = width / steps.x,	y = height / steps.y }
	rot		= {
				{ 0,     0,      0      },
				{ pi/2,  0,      size.y },
				{ pi,    size.x, size.y },
				{3*pi/2, size.x, 0      }
				}
	qd, cell, r = {}, {}, 1		-- r is default rotation (0) via rot[1][1]
	for j = 1, steps.y do
		qd[j], cell[j] = {}, {}
		for i = 1, steps.x do
			qd[j][i] = gr.newQuad( size.x * (i-1), size.y * (j-1), size.x, size.y, width, height )
			cell[j][i] = { j, i, r }	-- cell contains coordinates & rotation info
			end
		end
	end

function show_selection()
	gr.setColor( 255, 0, 0, 255 )
	draw_box( locate_cell( mouse ) )
	if sel then
		gr.setColor( 0, 255, 0, 255 )
		draw_box( locate_cell( sel ) )
		end
	gr.setColor( 255, 255, 255, 255 )
	end

function locate_cell ( m )	-- translate screen coordinates to grid location
	return 1 + math.floor( m[1] / size.x ), 1 + math.floor( m[2] / size.y )
	end

function grid()
	gr.setColor( 255, 255, 255, 55 )
	for y = -1, steps.y  do
		for x = -1, steps.x  do
			draw_box( x, y )
			end
		end
	gr.setColor( 255, 255, 255, 255 )
	end

function draw_box( x, y )
	gr.rectangle( 'line', (x-1) * size.x, (y-1) * size.y, size.x, size.y )
	end

function randomize_quad_orientations()
	for j = 1, steps.y do
		for i = 1, steps.x do
			cell[j][i][3] = math.random( 1, #rot )
			end
		end
	end

function add_text()
	gr.setColor( 255, 255, 0, 255 )
	for i = 1, #text do
		gr.print( text[ i ], 445, 15 * i )
		end
	end

function shuffle_cells()
	for j = 1, steps.y do shuffle_row(j)	end
	for i = 1, steps.x do shuffle_column(i)	end
	end

function shuffle_row(r)
	local n = steps.x
	while n >= 2 do
		local k = math.random(n)
		qd[r][n], qd[r][k] = qd[r][k], qd[r][n]
		n = n - 1
		end
	end

function shuffle_column(c)
	local n = steps.y
	while n >= 2 do
		local k = math.random(n)
		qd[n][c], qd[k][c] = qd[k][c], qd[n][c]
		n = n - 1
		end
	end
